package com.bocloud.sms.booter.controller;

import com.auth0.jwt.interfaces.Claim;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.Tokens;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.HttpSession;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.Objects;
import java.util.Optional;

/**
 * @author tyl
 * {@code @time} 2020/4/24 周五 20:02
 */
@Slf4j
@Controller
@RequestMapping
@Tag(name = "跳转控制")
public class DispatchController {

    private final SystemConfigRepository configRepository;
    private final StringRedisTemplate redisTemplate;


    public DispatchController(SystemConfigRepository systemConfigService, StringRedisTemplate redisTemplate) {
        this.configRepository = systemConfigService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 单点登录跳转接口
     *
     * @param request  浏览器请求
     * @param response 浏览器响应
     * @throws Exception 出现的异常
     */

    @RequestMapping("/v1/redirect")
    @Operation(summary = "单点登录跳转")
    public void redirect(HttpServletRequest request, HttpServletResponse response) throws Exception {
        //获取登录地址
        String target = request.getParameter("target");
        String redirectParameter = request.getParameter("redirect");
        SystemConfig config;
        if (target == null || "Manager".equals(target)) {
            config = configRepository.queryByCode("ManagerAddress");
        } else {
            config = configRepository.queryByCode("TenantAddress");
        }
        String loginAddress = config.getValue();
        log.info("#######debuge########the request url is {}", request.getRequestURL() + "?" + request.getQueryString());
        log.info("#######debuge########the redirect url is {}", redirectParameter);
        String redirect = Optional.ofNullable(redirectParameter).orElse(loginAddress + "/#/");
        String token = request.getParameter("token");
        log.info("#######debuge########the redirect token is {}", token);
        Claim claim = Tokens.parse(token);
        if (!Objects.isNull(claim)) {
            String catalog = claim.asMap().get("catalog").toString();
            long uuid = Long.parseLong(claim.asMap().get(Common.UUID).toString());
            String tokenKey = Common.TOKEN + "_" + catalog + "_" + uuid;
            boolean checkResult = check(token, tokenKey);
            if (checkResult) {
                HttpSession session = request.getSession();
                session.setAttribute("redirect", redirect);
                session.setAttribute(Common.TOKEN, token);
                response.sendRedirect(loginAddress + "/#/sso");
            } else {
                String replace = redirect.replace("#", "%23");
                response.sendRedirect(loginAddress + "/#/login?redirect=" + replace);
            }
        } else {
            String replace = redirect.replace("#", "%23");
            response.sendRedirect(loginAddress + "/#/login?redirect=" + replace);
        }
    }

    private boolean check(String token, String tokenKey) {
        String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
        String oldTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_o");
        if (token.equals(newTokenCache)) {
            return true;
        } else {
            return token.equals(oldTokenCache);
        }
    }
}
