package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.GlobalManConfig;
import com.bocloud.sms.interfaces.GlobalManConfigService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author wpj
 * @since 2022/9/8
 */
@RestController
@RequestMapping("/v1/global/config")
@Slf4j
@Tag(name = "全局运维配置")
public class GlobalManConfigController {

    private final GlobalManConfigService globalManConfigService;

    public GlobalManConfigController(GlobalManConfigService globalManConfigService) {
        this.globalManConfigService = globalManConfigService;
    }


    @Operation(summary = "查询周期时间列表")
    @GetMapping
    public GeneralResult<GridBean<GlobalManConfig>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return globalManConfigService.list(pager, context);
    }


    @Operation(summary = "创建禁止周期时间")
    @PostMapping
    public Result create(@RequestBody GlobalManConfig globalManConfig,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return globalManConfigService.create(globalManConfig, requestContext);
    }

    @Operation(summary = "修改禁止周期时间")
    @PutMapping("/{id}")
    public Result modify(@RequestBody GlobalManConfig globalManConfig,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return globalManConfigService.modify(globalManConfig, requestContext);
    }


    @Operation(summary = "删除禁止周期时间")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return globalManConfigService.remove(id, requestContext);
    }


    @Operation(summary = "查询当前全局配置是否被禁止")
    @GetMapping("/isAllow")
    public Result isAllow(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return globalManConfigService.allow(requestContext);
    }

}
