package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.NameRule;
import com.bocloud.sms.interfaces.NameRuleService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 命名规则控制器
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/7/2
 */
@RestController
@RequestMapping("/v1/namings")
@Tag(name = "命名规则")
public class NameRuleController {

    private final NameRuleService nameRuleService;

    public NameRuleController(NameRuleService nameRuleService) {
        this.nameRuleService = nameRuleService;
    }

    /**
     * 获取命名规则列表
     *
     * @param pager 分页参数
     * @return 命名规则列表
     */
    @Operation(summary = "获取命名规则列表")
    @GetMapping
    public GeneralResult<GridBean<NameRule>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nameRuleService.list(pager, context);
    }

    /**
     * 添加命名规则
     *
     * @param nameRule 命名规则
     * @param context  上下文
     * @return 添加结果
     */
    @Operation(summary = "添加命名规则")
    @PostMapping
    public GeneralResult<NameRule> create(@RequestBody NameRule nameRule,
                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nameRuleService.create(nameRule, context);
    }

    /**
     * 修改命名规则
     *
     * @param id       命名规则id
     * @param nameRule 命名规则
     * @param context  上下文
     * @return 修改结果
     */
    @Operation(summary = "修改命名规则")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody NameRule nameRule, @Value(Common.REQ_CONTEXT) RequestContext context) {
        nameRule.setId(id);
        return nameRuleService.modify(nameRule, context);
    }

    /**
     * 根据命名规则生成云主机名称
     *
     * @param id        命名规则id
     * @param tenantId  租户id
     * @param userId    用户id
     * @param projectId 项目id
     * @return 云主机名称
     */
    @Operation(summary = "根据命名规则生成云主机名称")
    @PatchMapping("/{id}")
    public GeneralResult<String> patch(@PathVariable(value = Common.ID) Long id,
                                       @RequestParam(value = "tenantId") Long tenantId,
                                       @RequestParam(value = "userId") Long userId,
                                       @RequestParam(value = "projectId") Long projectId, RequestContext context) {
        return nameRuleService.generate(id, tenantId, userId, projectId, context);
    }

    /**
     * 删除命名规则
     *
     * @param id 命名规则id
     * @return 删除结果
     */
    @Operation(summary = "删除命名规则")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return nameRuleService.remove(id);
    }

    /**
     * 查看命名规则详情
     *
     * @param id 命名规则id
     * @return 命名规则
     */
    @Operation(summary = "查看命名规则详情")
    @GetMapping("/{id}")
    public GeneralResult<NameRule> detail(@PathVariable(value = Common.ID) Long id) {
        return nameRuleService.detail(id);
    }

}
