package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.PortalPanel;
import com.bocloud.sms.interfaces.PortalPanelService;
import com.bocloud.sms.model.PortalPanelBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 首页配置controller
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月27日
 */
@RestController
@RequestMapping("/v1/portal/panels")
@Tag(name = "主页配置管理")
public class PortalPanelController {

    @Autowired
    private PortalPanelService portalPanelService;

    /**
     * 查询配置列表展示
     *
     * @param context 上下文
     * @return 配置列表
     */
    @Operation(summary = "主页配置列表展示")
    @GetMapping
    public GeneralResult list(@RequestParam(value = "module", required = false) String module,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return portalPanelService.list(module, context);

    }

    /**
     * 更新首页配置
     *
     * @param portalPanelBean 首页配置
     * @param context         上下文
     * @return 更新结果
     */
    @Operation(summary = "保存主页配置")
    @PatchMapping
    public GeneralResult save(@RequestBody PortalPanelBean portalPanelBean,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != portalPanelBean) {
            if (portalPanelBean.getConfig() == null) {
                return new GeneralResult(false, "参数格式不正确");
            }
            List<PortalPanel> portalPanels = JSONObject.parseArray(portalPanelBean.getConfig(), PortalPanel.class);
            return portalPanelService.save(portalPanels, context.getTarget(), context.getTenant(), null, portalPanelBean.getModule());
        } else {
            return new GeneralResult(false, "参数格式不正确");
        }
    }

    /**
     * 恢复默认设置
     *
     * @param context 上下文
     * @return 恢复结果
     */
    @Operation(summary = "恢复默认设置")
    @PutMapping("/{module}/reset")
    public GeneralResult reset(@PathVariable(value = "module") String module, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return portalPanelService.reset(module, context);
    }

}
