package com.bocloud.sms.booter.scheduler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.Message;
import com.bocloud.sms.repository.MessageRepository;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * 从运控获取工单数据到站内信
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class YunKongMessageSchedule {

    @Value("${message.enable:false}")
    private Boolean enable;
    @Value("${message.host:http://127.0.0.1}")
    private String host;
    @Value("${message.reqUserId:0}")
    private String reqUserId;
    @Value("${message.systemId:0}")
    private String systemId;
    @Value("${message.userId:0}")
    private String userId;

    private final MessageRepository messageRepository;

    @Scheduled(cron = "${message.schedule:0 0/2 * * * ?}")
    protected void schedule() {
        if (!enable) return;

        Map<String, Object> params = new HashMap<>();
        params.put("page", 1);
        params.put("limit", 15);
        params.put("approve_status", 1);
        params.put("userId", userId);
        params.put("taskStatus", "");
        params.put("taskName", "");

        Map<String, Object> request = new HashMap<>();
        request.put("reqUserId", reqUserId);
        request.put("systemId", systemId);
        request.put("method", "/activiti/rwMainTask/getTaskList");
        request.put("params", params);

        String url = host + "/system/common/queryYKInterface";
        HttpClient httpClient = new HttpClient(PostDataFormat.RAW);
        log.info("【运控接口】curl -k -X POST -H 'Content-Type: application/json' -d '{}' '{}'", JSON.toJSONString(request), url);
        Result result = httpClient.post(new HashMap<>(), request, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());

        JSONArray array = JSON.parseObject(result.getMessage()).getJSONObject("data").getJSONArray("data");
        List<Message> oldMessages = messageRepository.list(Message.class, "select * from message where external_id is not null");
        List<Message> existsMessages = new ArrayList<>();

        for (int i = 0; i < array.size(); i++) {
            JSONObject item = array.getJSONObject(i);
            String externalId = item.getString("record_id");
            if (StringUtils.isEmpty(externalId)) {
//                log.info("【运控接口】当前外部id为null：" + item.toJSONString());
                continue;
            }
            Message message = oldMessages.stream()
                    .filter(m -> externalId.equals(m.getExternalId()))
                    .findFirst().orElse(new Message());
            message.setCatalog(RequestContext.Catalog.User);
            message.setStatus("UNREAD");
            message.setGmtCreate(new Date());
            message.setCreatorId(1L);
            message.setOwnerId(1L);
            message.setSender(1L);
            message.setExternalId(externalId);
            message.setName(item.getString("task_name"));
            message.setContent(item.getString("task_content"));
            if (message.getId() == null) {
                messageRepository.save(message);
            } else {
                messageRepository.update(message);
                existsMessages.add(message);
            }
        }

        oldMessages.removeAll(existsMessages);
        for (Message m : oldMessages) {
            messageRepository.remove(m.getId(), 1L);
        }
    }
}
