package com.bocloud.sms.booter.websocket.message;

import com.alibaba.fastjson.JSON;
import com.bocloud.sms.booter.websocket.model.CommonMessage;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext.Catalog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.*;

import java.io.IOException;
import java.util.Vector;

/**
 * 通知类WebSocket处理器
 *
 * @author dmw
 */
@Slf4j
@Component
public class MessageHandler implements WebSocketHandler, InitializingBean {


    private Vector<WebSocketSession> webSocketSessions;

    /*
     * (non-Javadoc) WebSocket连接建立之后的操作
     *
     * @see org.springframework.web.socket.WebSocketHandler#
     * afterConnectionEstablished(org.springframework.web.socket. WebSocketSession)
     */
    @Override
    public void afterConnectionEstablished(WebSocketSession webSocketSession) throws Exception {
        if (null == webSocketSession.getAttributes().get("catalog")
                || null == webSocketSession.getAttributes().get("id")) {
            log.error("Target Information Is Null !!");
            throw new Exception("Target Information Is Null !!");
        }
        this.webSocketSessions.addElement(webSocketSession);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.springframework.web.socket.WebSocketHandler#handleMessage(org.
     * springframework.web.socket.WebSocketSession,
     * org.springframework.web.socket.WebSocketMessage)
     */
    @Override
    public void handleMessage(WebSocketSession webSocketSession, WebSocketMessage<?> message) throws Exception {
        if (webSocketSession.isOpen() && "HeartBeat".equals(message.getPayload().toString())) {
            OperateResult result = new OperateResult();
            result.setData("{}");
            result.setSuccess(true);
            result.setOperate(message.getPayload().toString());
            CommonMessage content = new CommonMessage(JSON.toJSONString(result));
            webSocketSession.sendMessage(new TextMessage(content.toString()));
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.springframework.web.socket.WebSocketHandler#handleTransportError(org.
     * springframework.web.socket.WebSocketSession, java.lang.Throwable)
     */
    @Override
    public void handleTransportError(WebSocketSession webSocketSession, Throwable exception) throws Exception {
        if (webSocketSession.isOpen()) {
            webSocketSession.close();
        }
        this.webSocketSessions.remove(webSocketSession);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.springframework.web.socket.WebSocketHandler#afterConnectionClosed(org
     * .springframework.web.socket.WebSocketSession,
     * org.springframework.web.socket.CloseStatus)
     */
    @Override
    public void afterConnectionClosed(WebSocketSession webSocketSession, CloseStatus closeStatus) throws Exception {
        this.webSocketSessions.remove(webSocketSession);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.springframework.web.socket.WebSocketHandler#supportsPartialMessages()
     */
    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    /**
     * 关闭用户websocket的session连接
     *
     * @param targetId
     */
    public void closeUserSession(Long targetId, String targetCatalog) {
        for (int i = this.webSocketSessions.size() - 1; i >= 0; i--) {
            WebSocketSession webSocketSession = webSocketSessions.get(i);
            String catalog = (String) webSocketSession.getAttributes().get("catalog");
            Long id = (Long) webSocketSession.getAttributes().get("id");
            if (null == catalog || null == id) {
                continue;
            }
            if (id.equals(targetId) && catalog.equalsIgnoreCase(targetCatalog)) {
                if (webSocketSession.isOpen()) {
                    try {
                        webSocketSession.close();
                        webSocketSessions.remove(webSocketSession);
                        log.info("websocket session closed");
                    } catch (IOException e) {
                        log.error("close websocket session exception:", e);
                    }
                }
            }
        }
    }

    public void sendMessageToUsers(CommonMessage message) {
        for (WebSocketSession webSocketSession : this.webSocketSessions) {
            try {
                if (webSocketSession.isOpen()) {
                    webSocketSession.sendMessage(new TextMessage(message.toString()));
                }
            } catch (IOException e) {
                log.error("Send WebSocket Message To All Users Exception {}", e.getMessage());
            }
        }
    }

    /**
     * 发送websocket消息到指定用户ID
     *
     * @param userId  用户ID
     * @param message 消息内容
     */
    public void sendMessageToUser(Catalog targetCatalog, Long userId, CommonMessage message) {
        for (WebSocketSession webSocketSession : webSocketSessions) {
            String catalog = (String) webSocketSession.getAttributes().get("catalog");
            Long id = (Long) webSocketSession.getAttributes().get("id");
            if (id.equals(userId) && (Catalog.Tenant != targetCatalog || Catalog.Tenant.name().equalsIgnoreCase(catalog))) {
                try {
                    if (webSocketSession.isOpen()) {
                        webSocketSession.sendMessage(new TextMessage(message.toString()));
                        log.info("Send WebSocket Message to [{} {}] Success ！！！", catalog, userId);
                    } else {
                        log.warn("Send WebSocket Message to [{} {}]  Exception for session is closed ！！！", catalog,
                                userId);
                    }
                } catch (IOException e) {
                    log.error("Send WebSocket Message to [{} {}] Exception:{}", catalog, userId, e.getMessage());
                }
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.webSocketSessions = new Vector<>();
    }

}
