package com.bocloud.sms.service;

import com.bocloud.sms.interfaces.SysDateViewService;
import com.bocloud.sms.repository.BusinessRepository;
import com.bocloud.sms.repository.ProjectRepository;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Result;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: SysDateViewImpl
 * @Description:
 * @Author lxy
 * @Date 2021/9/29 10:10
 */
@Service("sysDateViewService")
@Slf4j
public class SysDateViewImpl implements SysDateViewService {

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private TenantRepository tenantRepository;
    @Autowired
    private BusinessRepository businessRepository;
    @Autowired
    private ProjectRepository projectRepository;


    @Override
    public Result systemCount() {
        try {
            List<Param> userParams = Lists.newArrayList();
            userParams.add(new Param(MapTools.simpleMap("status", "NORMAL"), Sign.EQ));
            int userNum = userRepository.count(userParams, Lists.newArrayList(), null);
            List<Param> tenantParams = Lists.newArrayList();
            tenantParams.add(new Param(MapTools.simpleMap("status", "NORMAL"), Sign.EQ));
            int tenantNum = tenantRepository.count(tenantParams);
            int businessNum = businessRepository.count(Lists.newArrayList());
            int projectNum = projectRepository.count(Lists.newArrayList());
            Map<String, Integer> result = new HashMap<>();
            //由于user中存在admin用户，系统页面查不到admin用户，所以直接将admin去除
            result.put("userNum", userNum - 1);
            result.put("tenantNum", tenantNum);
            result.put("businessNum", businessNum);
            result.put("projectNum", projectNum);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("systemCount select:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
