package com.bocloud.sms.service.monitor;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.GeneralResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import javax.sql.DataSource;
import java.sql.*;

/**
 * 告警配置资源类型
 *
 * @author fanzhenxi
 * @version 5.6.0
 * @date 2022/3/28 14:48
 */
@Configuration
@Slf4j
public class MySqlConfiguration {

    @Autowired
    private DataSource dataSource;

    public MySqlConfiguration() {
    }

    public GeneralResult dbStatus() {
        JSONObject sqlInfo = null;
        try {
            sqlInfo = this.getMySQLInfo((DruidDataSource) this.dataSource);
            return new GeneralResult<>(true, sqlInfo, "ok");
        } catch (Exception var2) {
            log.error("get datasource error", var2);
            return new GeneralResult<>(false, sqlInfo, "error");
        }
    }

    private JSONObject getMySQLInfo(DruidDataSource dataSource) {
        JSONObject mysql = new JSONObject();
        DruidPooledConnection conn;
        String host1;
        try {
            String host = dataSource.getUrl();
            conn = dataSource.getConnection();
            host1 = host.split("//?")[1].split("/")[0];
        } catch (Exception var20) {
            log.error("get data base connection error:" + var20);
            return mysql;
        }
        JSONArray slaveInfos = this.showSlaveStatus(conn);
        if (!slaveInfos.isEmpty()) {
            JSONObject slaveInfo = slaveInfos.getJSONObject(0);
            String masterHost = slaveInfo.getString("Master_Host") + ":" + slaveInfo.getString("Master_Port");
            String host2 = "";
            JSONArray mysqlInfos = new JSONArray();
            JSONArray masterHosts = new JSONArray();
            if (slaveInfo.getString("Master_Host") == null) {
                mysql.put("mode", "standalone");
                masterHosts.add(host1);
            } else {
                mysql.put("mode", "replication");
                masterHosts.add(masterHost);
                DruidDataSource source = new DruidDataSource();
                String url = dataSource.getUrl();
                String[] urls = url.split("//");
                source.setUrl(urls[0] + "//" + masterHost + "/" + urls[1].split("/")[1]);
                source.setUsername(dataSource.getUsername());
                source.setMaxWait(dataSource.getMaxWait());
                source.setPassword(dataSource.getPassword());
                JSONObject mysqlInfo2 = null;
                String host = "";
                try {
                    DruidPooledConnection connection = source.getConnection();
                    mysqlInfo2 = this.buildMySQLInfo(connection);
                    mysqlInfo2.put("host", masterHost);
                    JSONObject slaveStatusEntityObject = mysqlInfo2.getJSONArray("SlaveStatusEntity").getJSONObject(0);
                    host = slaveStatusEntityObject.getString("Master_Host") + ":" + slaveStatusEntityObject.getString("Master_Port");
                    connection.close();
                } catch (SQLException var19) {
                    log.error("get data base connection error:" + var19);
                }
                if (mysqlInfo2 != null && mysqlInfo2.getJSONArray("SlaveStatusEntity").getJSONObject(0).getString("Master_Host") != null) {
                    host2 = host;
                    masterHosts.add(host);
                }
                mysqlInfos.add(mysqlInfo2);
            }
            JSONObject mysqlInfo = this.buildMySQLInfo(conn);
            mysqlInfo.put("host", "".equals(host2) ? host1 : host2);
            mysqlInfos.add(mysqlInfo);
            mysql.put("mysqlInfos", mysqlInfos);
            mysql.put("masterHosts", masterHosts);
        } else {
            JSONArray mysqlInfos = new JSONArray();
            JSONArray masterHosts = new JSONArray();
            mysql.put("mode", "standalone");
            masterHosts.add(host1);
            JSONObject mysqlInfo = this.buildMySQLInfo(conn);
            mysqlInfo.put("host", host1);
            mysqlInfos.add(mysqlInfo);
            mysql.put("mysqlInfos", mysqlInfos);
            mysql.put("masterHosts", masterHosts);
        }
        try {
            conn.close();
        } catch (SQLException var18) {
            log.warn("close connection error:" + var18);
        }
        return mysql;
    }

    private JSONObject buildMySQLInfo(Connection connection) {
        JSONObject mysqlInfo = new JSONObject();
        mysqlInfo.put("ThreadsGlobalStatus", this.getThreadsStatus(connection));
        return mysqlInfo;
    }

    private JSONArray showSlaveStatus(Connection connection) {
        String sql = "show slave status;";
        return this.executeSQL(sql, connection);
    }

    private JSONArray getThreadsStatus(Connection connection) {
        String sql = "show global status like 'Threads_%%'";
        return this.executeSQL(sql, connection);
    }

    private JSONArray executeSQL(String sql, Connection conn) {
        JSONArray jsonArray = new JSONArray();
        try {
            PreparedStatement statement = conn.prepareStatement(sql);
            ResultSet resultSet = statement.executeQuery();
            while (resultSet.next()) {
                JSONObject rowObject = new JSONObject();
                ResultSetMetaData metaData = resultSet.getMetaData();
                for (int i = 1; i <= metaData.getColumnCount(); ++i) {
                    String columnName = metaData.getColumnName(i);
                    String resultString = resultSet.getString(i);
                    rowObject.put(columnName, resultString);
                }
                jsonArray.add(rowObject);
            }
            resultSet.close();
            statement.close();
        } catch (SQLException var11) {
            log.info("execute sql error:", var11);
        }
        return jsonArray;
    }
}
