package com.bocloud.sms.service.monitor;


import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.IOException;

/**
 * RabbitMQ监控类
 *
 * @author tyl
 * @since 2019/11/19 周二 15:46
 */
@Slf4j
public class RabbitMqMonitor {

    /**
     * 获取监控概览信息
     *
     * @param host     IP
     * @param username rabbitmq登录名
     * @param password rabbitmq登录密码
     */
    public static JSONObject getOverview(String host, String username, String password) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/overview";
        return getJsonObjectData(url, username, password);
    }

    /**
     * 获取单个节点信息
     *
     * @param host     ip
     * @param username 用户名
     * @param password 密码
     * @param nodeName 节点名
     * @return 节点信息
     */
    public static JSONObject getNodeMonitor(String host, String username, String password, String nodeName) {
        String hostPort = "http://" + host + ":15672";
        String url = hostPort + "/api/nodes/" + nodeName;
        return getJsonObjectData(url, username, password);
    }


    /**
     * 指定url获取Json数据
     *
     * @param url      url
     * @param username 登录用户名
     * @param password 登录密码
     * @return Json数据
     */
    private static JSONObject getJsonObjectData(String url, String username, String password) {
        HttpGet httpGet = new HttpGet(url);
        //设置超时时间为1s
        RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(3000).setConnectionRequestTimeout(1000).setSocketTimeout(3000).build();
        httpGet.setConfig(requestConfig);
        //Header添加权限校验 （需要密码登录）
        UsernamePasswordCredentials usernamePassword = new UsernamePasswordCredentials(username, password);
        httpGet.addHeader(BasicScheme.authenticate(usernamePassword, "UTF-8", false));
        httpGet.setHeader("Content-Type", "application/json");
        JSONObject jsonObject = null;
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        try {
            httpClient = HttpClients.createDefault();
            response = httpClient.execute(httpGet);
            String data;
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                HttpEntity entity = response.getEntity();
                if (entity != null) {
                    data = EntityUtils.toString(entity);
                    jsonObject = JSONObject.parseObject(data);
                }
            }
        } catch (IOException e) {
            log.error("get rabbitmq info error: {}", e.getMessage(), e);
        } finally {
            try {
                if (null != response) {
                    response.close();
                }
                if (null != httpClient) {
                    httpClient.close();
                }
            } catch (IOException e) {
                log.error("httpClient close failed: {}", e.getMessage(), e);
            }
        }
        return jsonObject;
    }

}
