package com.bocloud.sms.entity;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * 系统升级历史
 *
 * @author luyanchao
 * @date 2021-06-15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("upgrade_history")
public class UpgradeHistory extends GenericEntity {
    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 源数据源连接url
     */
    @Column("url")
    private String url;
    /**
     * 源数据源用户名
     */
    @Column("user_name")
    private String user;
    /**
     * 源数据源密码
     */
    @Column("pass")
    private String pass;
    /**
     * 执行结果
     */
    @Column("result")
    private String result;
    /**
     * 开始时间
     */
    @Column("start_time")
    private Date startTime;
    /**
     * 结束时间
     */
    @Column("end_time")
    private Date endTime;
    /**
     * 执行人
     */
    @Column("creator")
    @IgnoreAll
    private String creator;
    /**
     * 总任务数
     */
    @IgnoreAll
    private Integer total;
    /**
     * 已完成任务数
     */
    @IgnoreAll
    private Integer complete;
    /**
     * 升级任务列表
     */
    @IgnoreAll
    private List<UpgradeTask> tasks;

    public Long getUsed() {
        if (startTime == null) {
            return null;
        }
        if (endTime == null) {
            return System.currentTimeMillis() - startTime.getTime();
        }
        return endTime.getTime() - startTime.getTime();
    }

    public Integer getTotalCnt() {
        return this.total;
    }

    public Integer getCompleteCnt() {
        return this.complete;
    }
}
