package com.bocloud.sms.repository;

import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.bocloud.sms.entity.AccessLog;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 运行日志服务实现类
 *
 * @author tyl
 */
@Service("logRepository")
public class LogRepository extends BasicGenericDao<AccessLog, Long> {

    @Autowired
    public LogRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 获取审计日志列表
     *
     * @param page   页码
     * @param rows   行数
     * @param params 查询参数
     * @param sorter 排序参数
     * @return
     * @throws Exception
     */
    public List<AccessLog> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from access_log a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(AccessLog.class, sql);
    }

    /**
     * 获取审计日志数量
     *
     * @param params 查询参数
     * @return
     * @throws Exception
     */
    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from access_log where 1 = 1";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    /**
     * 批量插入审计日志
     *
     * @param logs 审计日志集合
     * @throws Exception
     */
    public void batch(List<AccessLog> logs) throws DataBaseException {
        this.batchSave(logs, logs.size());
    }

    /**
     * 删除审计日志
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     * @throws Exception
     */
    public boolean delete(String startTime, String endTime) throws DataBaseException {
        String sql = "delete from access_log where gmt_create between :startTime and :endTime ";
        Map<String, Object> paramMap = MapTools.simpleMap("startTime", startTime);
        paramMap.put("endTime", endTime);
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 获取审计日志列表
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     * @throws Exception
     */
    public List<Map<String, Object>> list(String startTime, String endTime) throws DataBaseException {
        String sql = "select a.target,a.action,a.response_ip,a.request_ip,a.gmt_create,a.module,a.cost,b.account,a.detail from access_log a left join users b on "
                + "a.user_id = b.id where a.gmt_create between :startTime and :endTime order by a.gmt_create ";
        Map<String, Object> params = MapTools.simpleMap("startTime", startTime);
        params.put("endTime", endTime);
        return this.list(sql, params);
    }

}
