package com.bocloud.sms.repository;

import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.bocloud.sms.entity.UpgradeHistory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 系统升级历史
 *
 * @author luyanchao
 * @date 2021-06-15
 */
@Repository("upgradeHistoryRepository")
public class UpgradeHistoryRepository extends JdbcGenericDao<UpgradeHistory, Long> {

    public UpgradeHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 分页查询系统升级历史
     *
     * @param pager 分页参数
     * @return 数据列表
     */
    public List<UpgradeHistory> list(Pager pager) {
        String sql = "select a.* from upgrade_history a where a.is_deleted = 0 ";
        Map<String, Object> params = getQueryBuilder().getParam(pager.getParams());
        getQueryBuilder().build(sql, pager, "a");
        return this.list(UpgradeHistory.class, sql, params);
    }

    /**
     * 查询系统升级历史数量
     *
     * @param params 查询参数
     * @return 数量
     */
    public int count(List<Param> params) {
        String sql = "select count(1) from upgrade_history a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 按id查询系统升级历史
     *
     * @param id 查询参数
     * @return 系统升级历史
     */
    public UpgradeHistory get(Long id) {
        String sql = "select a.* from upgrade_history a where a.is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = Collections.singletonMap("id", id);
        return this.list(UpgradeHistory.class, sql, paramMap).stream().findFirst().orElse(null);
    }

}
